@@ -2,7 +2,7 @@
  * \file delfile.c
  * Example program to delete a file off the device.
  *
- * Copyright (C) 2005-2007 Linus Walleij <triad@df.lth.se>
+ * Copyright (C) 2005-2008 Linus Walleij <triad@df.lth.se>
  * Copyright (C) 2006 Chris A. Debenham <chris@adebenham.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -20,13 +20,13 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <stdlib.h>
+#include <limits.h>
+
 #include "common.h"
 #include "string.h"
 #include "pathutils.h"
-
-void delfile_usage(void);
-void delfile_function(char *);
-void delfile_command(int, char **);
+#include "connect.h"
 
 extern LIBMTP_mtpdevice_t *device;
 extern LIBMTP_folder_t *folders;
@@ -37,10 +37,11 @@ void delfile_usage(void)
   printf("Usage: delfile [-n] <fileid/trackid> | -f <filename>\n");
 }
 
-void
+int
 delfile_function(char * path)
 {
-  int id = parse_path (path,files,folders);
+  uint32_t id = parse_path (path,files,folders);
+
   if (id > 0) {
     printf("Deleting %s which has item_id:%d\n",path,id);
     int ret = 1;
@@ -49,15 +50,20 @@ delfile_function(char * path)
       LIBMTP_Dump_Errorstack(device);
       LIBMTP_Clear_Errorstack(device);
       printf("Failed to remove file\n");
+      return 1;
     }
   }
+  return 0;
 }
 
-void delfile_command(int argc, char **argv)
+int delfile_command(int argc, char **argv)
 {
   int FILENAME = 1;
   int ITEMID = 2;
   int field_type = 0;
+  int i;
+  int ret = 0;
+
   if ( argc > 2 ) {
     if (strncmp(argv[1],"-f",2) == 0) {
       field_type = FILENAME;
@@ -67,16 +73,17 @@ void delfile_command(int argc, char **argv)
       strcpy(argv[1],"0");
     } else {
       delfile_usage();
-      return;
+      return 0;
     }
   } else {
     delfile_usage();
-    return;
+    return 0;
   }
-  int i;
+
   for (i=1;i<argc;i++) {
-    int id;
+    uint32_t id;
     char *endptr;
+
     if (field_type == ITEMID) {
       // Sanity check song ID
       id = strtoul(argv[i], &endptr, 10);
@@ -91,7 +98,6 @@ void delfile_command(int argc, char **argv)
         id = 0;
       }
     }
-    int ret = 0;
     if (id > 0 ) {
       printf("Deleting %s\n",argv[i]);
       ret = LIBMTP_Delete_Object(device, id);
@@ -103,5 +109,6 @@ void delfile_command(int argc, char **argv)
       ret = 1;
     }
   }
+  return ret;
 }
 
