@@ -29,8 +29,8 @@
 #ifndef LIBMTP_H_INCLUSION_GUARD
 #define LIBMTP_H_INCLUSION_GUARD
 
-#define LIBMTP_VERSION 1.1.13
-#define LIBMTP_VERSION_STRING "1.1.13"
+#define LIBMTP_VERSION 1.1.17
+#define LIBMTP_VERSION_STRING "1.1.17"
 
 /* This handles MSVC pecularities */
 #ifdef _MSC_VER
@@ -409,7 +409,7 @@ typedef enum {
  */
 typedef enum {
   /**
-   * This capability tells whether you can call the funcion getting
+   * This capability tells whether you can call the function getting
    * partial objects, @see LIBMTP_GetPartialObject()
    */
   LIBMTP_DEVICECAP_GetPartialObject,
@@ -426,6 +426,16 @@ typedef enum {
    * @see LIBMTP_TruncateObject()
    */
   LIBMTP_DEVICECAP_EditObjects,
+  /**
+   * This capability tells whether you can call the function to
+   * move an object, @see LIBMTP_MoveObject()
+   */
+  LIBMTP_DEVICECAP_MoveObject,
+  /**
+   * This capability tells whether you can call the function to
+   * copy an object, @see LIBMTP_CopyObject()
+   */
+  LIBMTP_DEVICECAP_CopyObject,
 } LIBMTP_devicecap_t;
 
 /**
@@ -800,6 +810,7 @@ enum LIBMTP_event_enum {
   LIBMTP_EVENT_STORE_REMOVED,
   LIBMTP_EVENT_OBJECT_ADDED,
   LIBMTP_EVENT_OBJECT_REMOVED,
+  LIBMTP_EVENT_DEVICE_PROPERTY_CHANGED,
 };
 typedef enum LIBMTP_event_enum LIBMTP_event_t;
 
@@ -1038,6 +1049,8 @@ int LIBMTP_Set_Album_Name(LIBMTP_mtpdevice_t *, LIBMTP_album_t *, const char *);
  * @{
  */
 int LIBMTP_Delete_Object(LIBMTP_mtpdevice_t *, uint32_t);
+int LIBMTP_Move_Object(LIBMTP_mtpdevice_t *, uint32_t, uint32_t, uint32_t);
+int LIBMTP_Copy_Object(LIBMTP_mtpdevice_t *, uint32_t, uint32_t, uint32_t);
 int LIBMTP_Set_Object_Filename(LIBMTP_mtpdevice_t *, uint32_t , char *);
 int LIBMTP_GetPartialObject(LIBMTP_mtpdevice_t *, uint32_t const,
                             uint64_t, uint32_t,
@@ -1058,6 +1071,13 @@ int LIBMTP_Read_Event(LIBMTP_mtpdevice_t *, LIBMTP_event_t *, uint32_t *);
 int LIBMTP_Read_Event_Async(LIBMTP_mtpdevice_t *, LIBMTP_event_cb_fn, void *);
 int LIBMTP_Handle_Events_Timeout_Completed(struct timeval *, int *);
 
+/**
+ * @}
+ * @defgroup custom Custom operations API.
+ * @{
+ */
+int LIBMTP_Custom_Operation(LIBMTP_mtpdevice_t *, uint16_t, int, ...);
+
 /** @} */
 
 /* End of C++ exports */
