@@ -130,7 +130,7 @@ static int usb_get_endpoint_status(PTP_USB* ptp_usb, int ep, uint16_t* status);
  *        successful.
  * @param numdevs a pointer to an integer that will hold the number
  *        of devices in the device list if the call was successful.
- * @return 0 if the list was successfull retrieved, any other
+ * @return 0 if the list was successful retrieved, any other
  *        value means failure.
  */
 int LIBMTP_Get_Supported_Devices_List(LIBMTP_device_entry_t ** const devices, int * const numdevs)
@@ -733,7 +733,7 @@ void dump_usbinfo(PTP_USB *ptp_usb)
 }
 
 /**
- * Retrieve the apropriate playlist extension for this
+ * Retrieve the appropriate playlist extension for this
  * device. Rather hacky at the moment. This is probably
  * desired by the managing software, but when creating
  * lists on the device itself you notice certain preferences.
@@ -855,7 +855,7 @@ ptp_read_func (
       // this is the last packet
       toread = size - curread;
       // this is equivalent to zero read for these devices
-      if (readzero && FLAG_NO_ZERO_READS(ptp_usb) && toread % 64 == 0) {
+      if (readzero && FLAG_NO_ZERO_READS(ptp_usb) && (toread % ptp_usb->inep_maxpacket == 0)) {
         toread += 1;
         expect_terminator_byte = 1;
       }
