@@ -38,6 +38,32 @@
 #include "libmtp.h"
 #include "util.h"
 
+
+/**
+ * This prints to stdout info about device being UNKNOWN, its
+ * ids, and libmtp's version number.
+ *
+ * @param dev_number the device number
+ * @param id_vendor vendor ID from the usb_device_desc struct
+ * @param id_product product ID from the usb_device_desc struct
+ */
+void device_unknown(const int dev_number, const int id_vendor, const int id_product)
+{
+  // This device is unknown to the developers
+  LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is UNKNOWN in libmtp v%s.\n",
+    dev_number,
+    id_vendor,
+    id_product,
+    LIBMTP_VERSION_STRING);
+  LIBMTP_ERROR("Please report this VID/PID and the device model to the "
+               "libmtp development team\n");
+  /*
+   * Trying to get iManufacturer or iProduct from the device at this
+   * point would require opening a device handle, that we don't want
+   * to do right now. (Takes time for no good enough reason.)
+   */
+}
+
 /**
  * This dumps out a number of bytes to a textual, hexadecimal
  * dump.
@@ -123,4 +149,3 @@ char *strndup (const char *s, size_t n)
   return ret;
 }
 #endif
-
